#ifndef GOOGLE_PROTOBUF_COMPILER_CPP_CPP_GENERATOR_H_
#define GOOGLE_PROTOBUF_COMPILER_CPP_CPP_GENERATOR_H_

#include <google/protobuf/compiler/cpp/generator.h>

#endif  // GOOGLE_PROTOBUF_COMPILER_CPP_CPP_GENERATOR_H_
